package com.nisum.ascend.promotions.controller;

import com.nisum.ascend.promotions.dto.PromotionDto;
import com.nisum.ascend.promotions.exception.PromotionNotFoundException;
import com.nisum.ascend.promotions.exception.PromotionAlreadyExistsException;
import com.nisum.ascend.promotions.model.Promotion;
import com.nisum.ascend.promotions.repository.PromotionRepository;
import com.nisum.ascend.promotions.service.PromotionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/api/promos")
public class PromotionsController {

    @Autowired
    PromotionService promotionService;

    @GetMapping()
    public ResponseEntity<Flux<PromotionDto>> getAllPromotions(@RequestParam(required = false) String sku){
        if (sku != null){
            return ResponseEntity.ok(promotionService.findPromotionsByProductSku(sku));
        }
        else return ResponseEntity.ok(promotionService.findAll());
    }

    @GetMapping("/{id}")
    public ResponseEntity<Mono<PromotionDto>> getPromotionById(@PathVariable String id){
        Mono<PromotionDto> responseData =  promotionService
                .findPromoById(id)
                .switchIfEmpty(Mono.error(new PromotionNotFoundException(id)));
        return ResponseEntity.ok().body(responseData);
    }

    @PostMapping("")
    public Mono<PromotionDto> createPromotion(@RequestBody Promotion newPromotion){
        return promotionService.createPromotion(newPromotion).onErrorMap(throwable -> new PromotionAlreadyExistsException());
    }

    @PostMapping("/bulkSearch")
    public ResponseEntity<Flux<PromotionDto>> bulkSearchPromotionsByItemSku(@RequestBody List<String> skus){
        return ResponseEntity.ok(promotionService.bulkSearchSku(skus));
    }

    @PutMapping("/{promoId}")
    public Mono<ResponseEntity<PromotionDto>> updatePromotionById(@PathVariable("promoId") String promoId, @RequestBody Promotion newPromotion){
        return promotionService.updatePromotion(promoId,newPromotion)
                .map(updatedProduct -> ResponseEntity.ok(updatedProduct))
                .defaultIfEmpty(ResponseEntity.badRequest().build());
    }


    @DeleteMapping("/{promoId}")
    public Mono<ResponseEntity<Void>> deletePromotionById(@PathVariable("promoId") String promoId){
        return promotionService.deletePromotion(promoId)
                .map(res -> ResponseEntity.ok().<Void>build())
                .defaultIfEmpty(ResponseEntity.notFound().build());


    }
}
